--------------------------------------------------------
-- Namespaces
--------------------------------------------------------
local MCL, core = ...;
core.Config = {};

local MCL_Load = core.Config;

--------------------------------------
-- Defaults (usually a database!)
--------------------------------------
local defaults = {
	theme = {
		r = 0, 
		g = 0.8, -- 204/255
		b = 1,
		hex = "00ccff"
	}
}

local MCL_collectableMounts = 0
local MCL_u = 0
local MCL_rowC = 0
local MCL_x = 10
local MCL_y = -25
local MCL_MF
local MCL_currentFrame
local MCL_totalCollected
local MCL_totalCollectedNoStore
local MCL_complete
local MCL_totalc
local MCL_total

local function InitMounts()
	local x = C_MountJournal.GetMountFromItem(186654)
	if x == nil then
		C_Timer.After(0, InitMounts)
		return
	end
	MCLFrame:Show()
end

function MCL_Load:Toggle()
	if not MCLFrame then
		MCL_Load:CreateMenu()
		InitMounts()
		return
	end
	MCLFrame:SetShown(not MCLFrame:IsShown());
end

function MCL_Load:GetThemeColor()
	local c = defaults.theme;
	return c.r, c.g, c.b, c.hex;
end
function MCL_Load:CreateIcons(point, xOffset, yOffset, id, expID, collected)
	local mount = CreateFrame("Button", nil, expID, "BackdropTemplate")
	mount:SetPoint(point, expID, xOffset, yOffset)
	mount:SetSize(38, 38)
	mount.text = mount:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	mount.text:SetFont("Fonts\\FRIZQT__.TTF", 10)
	mount.text:SetPoint("BOTTOM", mount, "CENTER", 0, 0)
	mount.tex = mount:CreateTexture()
	mount.tex:SetAllPoints(mount)
	mount.tex:SetTexture(GetItemIcon(id))
	if collected then
		mount:SetBackdrop({
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 39,
		})
		mount:SetBackdropBorderColor(0, 1, 0)
	else
		mount.tex:SetVertexColor(0.75, 0.75, 0.75, 0.3);	
	end
	--Adding a tooltip for mounts
	local item, itemLink = GetItemInfo(id);
	mount:HookScript("OnEnter", function()
		if (itemLink) then
			GameTooltip:SetOwner(mount, "ANCHOR_TOP")
			GameTooltip:SetHyperlink(itemLink)
			GameTooltip:Show()
			mount:SetHyperlinksEnabled(true)

		end
	end)
	mount:HookScript("OnLeave", function()
		GameTooltip:Hide()
	end)		
	return mount;
end
function MCL_Load:CreateMountIcons(point, xOffset, yOffset, expID, collected, icon, pos)
	local mount = CreateFrame("Button", nil, expID, "BackdropTemplate")
	mount:SetPoint(point, expID, xOffset, yOffset)
	mount:SetSize(38, 38)
	mount.text = mount:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	mount.text:SetFont("Fonts\\FRIZQT__.TTF", 10)
	mount.text:SetPoint("BOTTOM", mount, "CENTER", 0, 0)
	mount.tex = mount:CreateTexture()
	mount.tex:SetAllPoints(mount)
	mount.tex:SetTexture(icon)
	if collected then
		mount:SetBackdrop({
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 39,
		})
		mount:SetBackdropBorderColor(0, 1, 0)
	else
		mount.tex:SetVertexColor(0.75, 0.75, 0.75, 0.3);	
	end
	--Adding a tooltip for mounts
	return mount;
end
function MCL_Load:CategoryFrame(expT, relativeFrame, xOffset, yOffset)
	--Creating a frame to place expansion content in.
	local categorie = CreateFrame("Frame", nil, relativeFrame);
	categorie:SetSize(800, 90);
	categorie:SetPoint("TOPLEFT", relativeFrame,  xOffset, yOffset);

	--Creating a frame to place title of expansion
	categorie.title = categorie:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	categorie.title:SetPoint("TOPLEFT", 10, 0)
	categorie.title:SetText(expT)
	return categorie;
end
local function ScrollFrame_OnMouseWheel(self, delta)
	local newValue = self:GetVerticalScroll() - (delta * 50);
	
	if (newValue < 0) then
		newValue = 0;
	elseif (newValue > self:GetVerticalScrollRange()) then
		newValue = self:GetVerticalScrollRange();
	end
	
	self:SetVerticalScroll(newValue);
end

local function Tab_OnClick(self)
	PanelTemplates_SetTab(self:GetParent(), self:GetID());

	local scrollChild = MCL_MF.ScrollFrame:GetScrollChild();
	if(scrollChild) then
		scrollChild:Hide();
	end

	MCL_MF.ScrollFrame:SetScrollChild(self.content);
	self.content:Show();
	MCL_MF.ScrollFrame:SetVerticalScroll(0);
end

local function SetTabs(frame, numTabs, ...)
	frame.numTabs = numTabs;
	local contents = {};
	local frameName = frame:GetName();

	for i = 1, numTabs do

		local tab = CreateFrame("Button", frameName.."Tab"..i, frame, "CharacterFrameTabButtonTemplate");
		tab:SetID(i);
		tab:SetText(select(i, ...));
		tab:SetScript("OnClick", Tab_OnClick);

		tab.content = CreateFrame("Frame", nil, MCL_MF.ScrollFrame);
		tab.content:SetSize(760, 500);
		tab.content:Hide();

		table.insert(contents, tab.content);

		if (i == 1) then
			tab:SetPoint("TOPLEFT", MCL_MF, "BOTTOMLEFT", 5, 7);
		else
			tab:SetPoint("TOPLEFT", _G[frameName.."Tab"..(i-1)], "TOPRIGHT", -14, 0);
		end
	end

	Tab_OnClick(_G[frameName.."Tab1"]);

	return unpack(contents);
end

function MCL_Load:insertMountFromID(mountID, rowC, x, category, mountFrame, aFrame, isFactionSpecific, faction, isCollected, playerFaction)
	if (isFactionSpecific == true and faction == playerFaction) or isFactionSpecific == false then
		if MCL_rowC > 0 then
			MCL_x = MCL_x+48
		end
		MCL_rowC = MCL_rowC+1							
		mountFrame = self:CreateMountIcons("TOPLEFT", MCL_x, MCL_y, aFrame, isCollected, icon)
		MCL_collectableMounts = MCL_collectableMounts+1
	end
	if isCollected == true then
		MCL_totalc = MCL_totalc+1
		MCL_totalCollected = MCL_totalCollected+1
		if category ~= "Blizzard Store" then
			MCL_totalCollectedNoStore = MCL_totalCollectedNoStore+1
		end
	end
	MCL_complete = true
	return MCL_rowC, MCL_x, MCL_complete
end		

function MCL_Load:insertMountFromItem(ID, rowC, x, category, mountFrame, aFrame, mountID, isFactionSpecific, faction, isCollected, playerFaction)					
	if (isFactionSpecific == true and faction == playerFaction) or isFactionSpecific == false then
		if MCL_rowC > 0 then
			MCL_x = MCL_x+48
		end
		MCL_rowC = MCL_rowC+1
		MCL_u = MCL_u+1
		mountFrame = self:CreateIcons("TOPLEFT", MCL_x, MCL_y, ID, aFrame, isCollected)
		MCL_collectableMounts = MCL_collectableMounts+1												
		if isCollected == true then
			MCL_totalc = MCL_totalc+1
			MCL_totalCollected = MCL_totalCollected+1
			if category ~= "Blizzard Store" then
				MCL_totalCollectedNoStore = MCL_totalCollectedNoStore + 1
			end
		end
		local item, itemLink = GetItemInfo(mountID);
		mountFrame:SetScript("OnClick", function(self, button, down)
			if (itemLink) then
				print(itemLink)
			end
		end)
	--This will determine when to start a new row of mounts. Currently set to 6 mounts per row.								
		if(MCL_rowC == 50) then
			MCL_y = MCL_y-45
			MCL_x = 10
			MCL_rowC = 0
		end
		MCL_total = MCL_total+1
	end
	MCL_complete = true
	return MCL_rowC, MCL_x, MCL_complete		
end

function MCL_Load:CreateMenu()
    if not IsAddOnLoaded("Blizzard_Collections") then
        LoadAddOn("Blizzard_Collections")
    end
    MCL_MF = CreateFrame("Frame", "MCLFrame", UIParent, "BasicFrameTemplateWithInset");
    MCL_MF:Hide()
	--Master Frame for addon
	MCL_MF:SetMovable(true)
	MCL_MF:EnableMouse(true)
	MCL_MF:RegisterForDrag("LeftButton")
	MCL_MF:SetScript("OnDragStart", MCL_MF.StartMoving)
	MCL_MF:SetScript("OnDragStop", MCL_MF.StopMovingOrSizing)
	MCL_MF:SetFrameStrata("DIALOG")

	--MCL Frame settings
	MCL_MF:SetSize(1200, 600); -- width, height
	MCL_MF:SetPoint("CENTER", UIParent, "CENTER"); -- point, relativeFrame, relativePoint, xOffset, yOffset
	MCL_MF:EnableMouse(true)
	MCL_MF:SetHyperlinksEnabled(true)
	MCL_MF:SetScript("OnHyperlinkClick", ChatFrame_OnHyperlinkShow)

	--Creating title for frame
	MCL_MF.title = MCL_MF:CreateFontString(nil, "OVERLAY", "GameFontHighlight");
	MCL_MF.title:SetPoint("LEFT", MCL_MF.TitleBg, "LEFT", 5, 0);
	MCL_MF.title:SetText("Mount Collection Log");

	MCL_MF.ScrollFrame = CreateFrame("ScrollFrame", nil, MCL_MF, "UIPanelScrollFrameTemplate");
	MCL_MF.ScrollFrame:SetPoint("TOPLEFT", MCL_MF.Bg, "TOPLEFT", 4, -8);
	MCL_MF.ScrollFrame:SetPoint("BOTTOMRIGHT", MCL_MF.Bg, "BOTTOMRIGHT", -3, 4);
	MCL_MF.ScrollFrame:SetClipsChildren(true);
	MCL_MF.ScrollFrame:SetScript("OnMouseWheel", ScrollFrame_OnMouseWheel);	

	MCL_MF.ScrollFrame.ScrollBar:ClearAllPoints();
	MCL_MF.ScrollFrame.ScrollBar:SetPoint("TOPLEFT", MCL_MF.ScrollFrame, "TOPRIGHT", -12, -18);
	MCL_MF.ScrollFrame.ScrollBar:SetPoint("BOTTOMRIGHT", MCL_MF.ScrollFrame, "BOTTOMRIGHT", -7, 18);

	local MCL_playerFaction
	local MCL_playerFactionStr = UnitFactionGroup("player")
	if MCL_playerFactionStr == "Alliance" then
		MCL_playerFaction = 1
	else
		MCL_playerFaction = 0
	end

	MCL_totalCollected = 0
	MCL_totalCollectedNoStore = 0

	local MCL_classic, MCL_tbc, MCL_wotlk, MCL_cata, MCL_mop, MCL_wod, MCL_legion, MCL_bfa, MCL_sl, MCL_racial, MCL_professions, MCL_pvp, MCL_worldevents, MCL_promotion, MCL_other = SetTabs(MCL_MF, 15, "Classic", "TBC", "WOTLK", "CATA", "MOP", "WOD", "LEGION", "BFA", "SL", playerFactionStr, "Professions", "PVP", "WorldEvents", "Promotion", "Other");

	local MCL_sections = {MCL_classic, MCL_tbc, MCL_wotlk, MCL_cata, MCL_mop, MCL_wod, MCL_legion, MCL_bfa, MCL_sl, MCL_racial, MCL_professions, MCL_pvp, MCL_worldevents, MCL_promotion, MCL_other}
	local MCL_sectionStrings = {"Classic", "TBC", "WOTLK", "CATA", "MOP", "WOD", "LEGION", "BFA", "SL", playerFactionStr, "Professions", "PVP", "WorldEvents", "Promotion", "Other"}

-- update "frame" with new information to display
	local MCL_mountFrames = {}
	local MCL_a = {}
	--Creating a frame to place expansion content in.
	--For each value section and category run this loop
	local function LoadData()
		for k,v in pairs(core.sectionNames) do
			--Reset location of frames
			local yTwo = -20
			local xTwo = 20
			MCL_rowC = 0
			local frameName = MCL_MF:GetName()
			--Cycle through each section and check if database entry matches any string.
			for s = 1, #MCL_sectionStrings do
				if v.name == MCL_sectionStrings[s] then	
					--If string matches a section then we loop through all categories for that section and create a frame.	
					for i = 1, #v.category do
						MCL_currentFrame = self:CategoryFrame(v.category[i], MCL_sections[s], xTwo, yTwo)
						table.insert(MCL_a, MCL_currentFrame)
						MCL_rowC = 0
						MCL_x = 10
						MCL_y = -25
						for b,n in pairs(core.mountList) do
							if n.name == MCL_sectionStrings[s] then
								for h,j in pairs(n) do
									if j.name == v.category[i] then
										if j.mounts then
											MCL_total = 0
											MCL_totalc = 0
											for t = 1, #j.mountID do
												if j.mountID[t] then											
													local _, _, icon, _, _, _, _, isFactionSpecific, faction, _, isCollected, mountID = C_MountJournal.GetMountInfoByID(j.mountID[t])
													MCL_rowC, MCL_x, MCL_complete = self:insertMountFromID(j.mountID[t], MCL_rowC, MCL_x, v.category[i], MCL_mountFrames[MCL_u], MCL_currentFrame, isFactionSpecific, faction, isCollected, MCL_playerFaction)
												end					
											end													
											for t = 1, #j.mounts do
												if C_MountJournal.GetMountFromItem(j.mounts[t]) then
													local mountID = C_MountJournal.GetMountFromItem(j.mounts[t])
													local _, _, icon, _, _, _, _, isFactionSpecific, faction, _, isCollected, mountID = C_MountJournal.GetMountInfoByID(mountID)
													MCL_rowC, MCL_x, MCL_complete = self:insertMountFromItem(j.mounts[t], MCL_rowC, MCL_x, v.category[i], MCL_mountFrames[MCL_u], MCL_currentFrame, mountID, isFactionSpecific, faction, isCollected, MCL_playerFaction)	
												end								
											end
											if MCL_currentFrame then
												MCL_currentFrame.title:SetText(v.category[i].." ("..MCL_totalc.."/"..MCL_total..")")
											end
										end
									end
								end
							end
						end
						yTwo = yTwo - 80
					end
				end
			end
		end			
		MCL_MF.title:SetText("Mount Collection Log | Total Collected "..MCL_totalCollected.." | Total Without Store Mounts ".. MCL_totalCollectedNoStore.." | Total Collectable Mounts "..MCL_collectableMounts);
	end
	LoadData()
	MCL_MF:SetScript("OnShow", function()
		LoadData()
	end)
	return MCL_MF;
end